{% load url from future %}

{% for country in countrys %}
<a class="block choice" href="{% url 'admin:cities_light_country_change' country.pk %}">{{ country }}</a>
{% endfor %}
{% for region in regions %}
<a class="block choice" href="{% url 'admin:cities_light_region_change' region.pk %}">{{ region }}</a>
{% endfor %}
{% for city in cities %}
<a class="block choice" href="{% url 'admin:cities_light_city_change' city.pk %}">{{ city }}</a>
{% endfor %}
{% for group in groups %}
<a class="block choice" href="{% url 'admin:auth_group_change' group.pk %}">{{ group }}</a>
{% endfor %}
{% for user in users %}
<a class="block choice" href="{% url 'admin:auth_user_change' user.pk %}">{{ user }}</a>
{% endfor %}
